@@ -4,7 +4,7 @@ use criterion::{
 };
 use is_prime_for_primitive_int::IsPrime;
 use ring_algorithm::*;
-type Z = num::BigInt;
+type Z = rug::Integer;
 
 fn make_input(len: usize) -> (Vec<Z>, Vec<Z>) {
     let mut u = Vec::with_capacity(len);
@@ -25,11 +25,11 @@ fn make_input(len: usize) -> (Vec<Z>, Vec<Z>) {
 pub fn bench(c: &mut Criterion) {
     let plot_config = PlotConfiguration::default().summary_scale(AxisScale::Logarithmic);
     let mut group = c.benchmark_group("crt");
-    group.plot_config(plot_config.clone());
-    for i in 5..=17 {
+    group.plot_config(plot_config);
+    for i in 5..=15 {
         let size = 1 << i;
         let (u, m) = make_input(size);
-        if i <= 13 {
+        if i <= 11 {
             group.bench_with_input(BenchmarkId::new("crt", size), &size, |b, &_| {
                 b.iter(|| chinese_remainder_theorem(black_box(&u), black_box(&m)))
             });
